﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using CMS.DocumentEngine;
using CMS.Base;
using CMS.EventLog;
using CMS.Helpers;

[SlackIntegrationModuleLoader]
public partial class CMSModuleLoader
{
    /// <summary>
    /// Attribute class that ensures the loading of custom handlers.
    /// </summary>
    private class SlackIntegrationModuleLoader : CMSLoaderAttribute
    {
        private string strSlackURL = "[WEBHOOKURL]";
        private string strSlackUsername = "[SLACKUSERNAME]";
        private string strSlackChannel = "[SLACKCHANNEL]";

        /// <summary>
        /// The system executes the Init method of the CMSModuleLoader attributes when the application starts.
        /// </summary>
        public override void Init()
        {
            // Assigns custom handlers to events
            WorkflowEvents.CheckOut.After += CheckOut_After;
            WorkflowEvents.CheckIn.After += CheckIn_After;
            WorkflowEvents.UndoCheckOut.After += UndoCheckOut_After;
        }

        private void UndoCheckOut_After(object sender, WorkflowEventArgs e)
        {
            PostToSlack("Document <" + URLHelper.GetAbsoluteUrl(e.Document.NodeAliasPath) + "|" + e.Document.DocumentName + "> checked out undone.");
        }

        private void CheckIn_After(object sender, WorkflowEventArgs e)
        {
            PostToSlack("Document <" + URLHelper.GetAbsoluteUrl(e.Document.NodeAliasPath) + "|" + e.Document.DocumentName + "> checked in.");
        }

        private void CheckOut_After(object sender, WorkflowEventArgs e)
        {
            PostToSlack("Document <" + URLHelper.GetAbsoluteUrl(e.Document.NodeAliasPath) + "|" + e.Document.DocumentName + "> checked out.");
        }

        private void PostToSlack(string strMessage)
        {
            try
            {
                Custom.SlackClient client = new Custom.SlackClient(strSlackURL);
                client.PostMessage(username: strSlackUsername,
                    text: strMessage,
                    channel: strSlackChannel);
            }
            catch (Exception ex)
            {
                EventLogProvider.LogException("SlackIntegrationModuleLoader", "EXCEPTION", ex);
            }
        }

    }
}